<?php
// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Christian Stocker <chregu@bitflux.ch>                        |
// +----------------------------------------------------------------------+
//
// $Id: transformer.php,v 1.4 2003/07/09 19:32:48 glandium Exp $

include_once(PAPAAN_BASE.'/components/component.php');
/**
 * Base-class for all the transformers
 *
 * Transformers are used in <map:transform> in sitemap.xml and transform
 * an xml-document to another. 
 * Input and Output is therefore xml and passed by reference to the 
 * method DomStart()
 *
 * @author   Christian Stocker <chregu@bitflux.ch>
 * @version  $Id: transformer.php,v 1.4 2003/07/09 19:32:48 glandium Exp $
 * @package  papaan
 */

class transformer extends component {


    function transformer(&$sitemap, $config) {
	$this->component($sitemap, $config);
    }

    function DomStart(&$xml) {
    }

    
    /* CACHING STUFF */
    /* Note about transformers: Allthough 'transformer' implements 'cachable', there are transformers
       that are _not cachable_. 'Xforms' and 'Phpprocessor' should certainly not be cached in  */

    /**
     * Generate cacheKey
     *
     * Calls the method inherited from 'Component', adds significant data from map:parameter, if set.
     *
     * @param   array  attributes
     * @param   int    last cacheKey
     * @see     generateKeyDefault()
     */
    function generateKey($attribs, $keyBefore){
        //add params?
        if(!empty($this->params)) $attribs['params'] = $this->params;
        //call default method
        return($this->generateKeyDefault($attribs, $keyBefore));
    }

    /** Generate validityObject  
     *
     * This is common to all "readers", you'll find the same code there.
     * I'm thinking about making a method in the class component named generateValidityFile() or alike
     * instead of having the same code everywhere..
     *
     * @author Hannes Gassert <hannes.gassert@unifr.ch>
     * @see  checkvalidity()
     * @return  array  $validityObject contains the components attributes plus file modification time and time of last access.
     */
    function generateValidity(){
        $validityObject = $this->attribs;
        //add params?
        if(!empty($this->params)) $attribs['params'] = $this->params; 
        $src = $this->getAttrib('src'); 
        $validityObject['src'] = $src;     
        if($src){
            $validityObject['filemtime'] = filemtime($src);
            $validityObject['fileatime'] = fileatime($src);
        }
        return($validityObject);
    }

    /**
     * Check validity of a validityObject from cache:
     *
     * This implements only the most simple form: If there's no fresher version, take that from cache.
     * I guess we'll need some more refined criteria.. Parameters from map:parameter are taken into account, if present.
     *
     * @return  bool  true if the validityObject indicates that the cached version can be used, false otherwise.
     * @param   object  validityObject
     */
    function checkValidity($validityObject){
        //check params?
        if(!empty($this->params)){
            $params = $this->params; ksort($params);
            $c_params = $validityObject['params']; ksort($c_params);
        }
        else{ $params = null;
              $c_params = $params; //equal for sure
        }
        return($params == $c_params                &&
               isset($validityObject['src'])       &&
               isset($validityObject['filemtime']) &&
               file_exists($validityObject['src']) &&
               ($validityObject['filemtime'] >= filemtime($validityObject['src'])));
    }

}
