<?php

// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Christian Stocker <chregu@bitflux.ch>                        |
// +----------------------------------------------------------------------+
//
// $Id: papaan.php,v 1.12 2003/07/17 21:55:05 glandium Exp $

define('PAPAAN_VERSION', '0.3.0');

define('PAPAAN_STRING', 'Papaan v'.PAPAAN_VERSION);

define('PAPAAN_BASE', dirname(__FILE__));

include_once(PAPAAN_BASE.'/sitemap/sitemap.php');

/**
* We have different error levels in papaan, it's much like
*  the PHP error handling. With PEAR::raiseError you can
*  declare how severe your error is.
**/
/**
* Won't produce any output ever
* @const PAPAAN_ERROR_SILENT
* @access public
*/
define('PAPAAN_ERROR_SILENT', 	1);
/**
* Errors, which are not that grave
* Errors with that level won't disturb papaan. They are more
*  of an informational level
* @const PAPAAN_ERROR_NOTICE
* @access public
*/
define('PAPAAN_ERROR_NOTICE', 	2);
/**
* Errors, which should be avoided, but the script is able to run further
* Errors with that level could have bad effects on papaan, but normally don't
* @const PAPAAN_ERROR_WARNING
* @access public
*/
define('PAPAAN_ERROR_WARNING', 	4);
/**
* Errors, which stop the execution of papaan
* Errors with that level have severe impact on papaan and the script will be stoped
* @const PAPAAN_ERROR_WARNING
* @access public
*/
define('PAPAAN_ERROR_FATAL',   	8);

/**
* Can be used for old deprecated functions..
* @const PAPAAN_ERROR_DEPRECATED
* @access public
*/
define('PAPAAN_ERROR_DEPRECATED',   	16);

/**
* No Error-reporting at all....
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_NONE', 0);
/**
* Every error-reporting level
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_ALL', 31);
/**
* Default error-reporting level
* At the moment = PAPAAN_ERROR_ALL
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_DEFAULT', PAPAAN_ERROR_ALL);

$GLOBALS['_PAPAAN_errorLevel']= PAPAAN_ERROR_DEFAULT;


/**
* Class for doing everything in papaan :)
*
* @author   Christian Stocker <chregu@bitflux.ch>
* @version  $Id: papaan.php,v 1.12 2003/07/17 21:55:05 glandium Exp $
* @package  papaan2
*/
class papaan {

	/**
	* The Directory, where the cached sitemaps should be saved
	*
	* @var string
	*/
	var $cacheDir = './tmp/';

	/**
	* Error Level

	* It's a static variable so it used in every instance of papaan
	*
	* @var int
	*/
	var $errorLevel = PAPAAN_ERROR_DEFAULT;

	/**
    * Constructor
    *
	* If $sitemapFile (or more) is provided, the method run() is immediately called.
	*
    * @param string $sitemapFile the location of sitemap.xml, can be relativ
	* @param string $uri the uri of the call, optional (takes _SERVER['REQUEST_URI'] then)
	* @param array $options class variable options
	* @see run()
    * @access public
	*/
    function papaan ($sitemapFile = null, $uri = null, $options = array()) {

        if ((isset($options['outputCache']) && $options['outputCache']) || (isset($options['outputCacheCallback']) && $options['outputCacheCallback'] && call_user_func($options['outputCacheCallback'])) )
        {
            include_once(PAPAAN_BASE.'/sitemap/outputcache.php');
            $oc = new OutputCache();
            $oc->start();
            $options['outputCache'] = true;

        } else {
            $options['outputCache'] = false;
        }


		if ($sitemapFile !== null)
		{
			$sm = $this->run(realpath($sitemapFile),$uri,$options);
		}

        if ($sm->outputCache)
        {
            if (! isset($options['outputCacheExpire'])) {
                $options['outputCacheExpire'] = 3600;
            }
        	$oc->end($sm, $options['outputCacheExpire']);
        }
	}

	/**
	* runs the sitemap
	*
	* More docu, please
	*
    * @param string $sitemapFile the location of sitemap.xml, can be relativ
	* @param string $uri the uri of the call, optional (takes _SERVER['REQUEST_URI'] then)
	* @param array $options class variable options
	* @see papaan(), setOptions
    * @access public
	*/
	function run($sitemapFile, $uri = null, $options = array()) {
		$this->setOptions($options);
		$sm = new sitemap($sitemapFile,$uri,$options);
		if (! $sm->runSitemap()) {
			header("HTTP/1.0 404 Not Found");
			header("Status: 404 Not Found");
			echo "404 Not Found";
		}
		return $sm;
	}


	/**
	* Sets class options
	*
	* Sets options for a class. the array has to be an associative
	* array, where the key is a class-variable and the value is the
	* value. With this method, you can't set class-variable, which
	* were not defined before
	*
	* @param array $options associative array
    * @access public
	*/
	function setOptions($options) {
		foreach($options as $key => $value) {
			if (isset($this->$key)) {
				$this->$key = $value;
			}
		}
	}

	/**
	* Strips the starting xml processing instruction from a xml file
	*
	* This is needed, if we have short_open_tag = On, because then php
	* chokes on it. We don't need it if it's = Off.
	*
	* @param string $xml The XML Document as a string
	* @access public
	* @return string The XML without the PI
	*/
	function stripXmlPI($xml)
	{
		return preg_replace('#^<\?xml[^>]*\?>\s*#','',$xml);;
	}

	/**
	* Save a file to the filesystem
	*
	* @param string $content The content of the file
	* @param string $filename The filename
	* @return bool
	* @access public
	*/
	function saveFile($content,$filename)
	{
		$fd = fopen($filename,'w');
		if (!$fd) {
			papaan::raiseError('Could not save '.$filename.' (In '. __FILE__ .':'.__LINE__.')',PAPAAN_ERROR_FATAL);
			return false;
		}
		fwrite($fd,$content);
		fclose($fd);
		return True;
	}


    /**
    * Trigger a PEAR error
    *
    * To improve performances, the PEAR.php file is included dynamically.
    * The file is only included when an error is triggered. So, in most
    * cases, the file isn't included and perfs are much better.
    *
	* After raising a PEAR_Error we call papaan::handleError(), which does
	*  the main papaan  errorhandling (printing and dying), if PEAR_Error didn't take
	*  any action before
	*
    * @param string $msg error message
    * @param int $code error code
	* @param string $file filename, where the error occured.
	* @param int  $line linenumber, where the error occured.
    * @param int $mode PEAR error mode, should not be used, since this is up to the user...
    * @access public
	* @return object PEAR_Error Object
    */
    function raiseError($msg, $code = null, $file = null, $line = null, $mode = null)
    {
        include_once('PEAR.php');
		if ($file !== null) {
	        $err = PEAR::raiseError(sprintf('%s [%s on line %d].', $msg, $file, $line), $code, $mode );
		} else {
		   $err = PEAR::raiseError(sprintf('%s', $msg), $code, $mode );
		}
		papaan::handleError($err);
		return $err;
    }

    /**
    * Tell whether a value is a PEAR error.
	*
	* This method can be used, so we don't have to load
	* PEAR.php just for checking this...
	*
    *
    * @param   mixed $data   the value to test
    * @access  public
    * @return  bool    true if parameter is an error
    */
    function isError($data)
	{
         return (bool)(is_object($data) &&
                      (get_class($data) == 'pear_error' ||
                      is_subclass_of($data, 'pear_error')));
    }

	/**
	* Handles Papaan errors
	*
	* This method does the error-handling of papaan
	*
	* papaan's error handling is based on PEAR's error handling.
	*  A papaan error is generated with papaan::raiseError(..), which
	*  on the other hand raises a PEAR_Error with PEAR::raiseError(..)
	*  This way, the user can use the PEAR errorhandling which is triggered
	*  at PEAR::raiseError(..). Therefore, if the user sets the
	*  PEAR::setErrorHandling for example to PEAR_ERROR_CALLBACK, he can
	*  intercept papaan-errors before we catch it here.
	*
	* If the PEAR::errorLevel is set to something different
	*  than the default value (PEAR_ERROR_RETURN) nothing won't
	*  happen here at all. Otherwise it will print the error messages
	*  and die if it's FATAL
	*
	* papaan::raiseError will automatically call this method, and therefore
	*  one doesn't have to check for errors within papaan.
	*
	* Performance in ErrorHandling is only important in isError, since we maybe
	*  use this function more often. raiseError and handleError will only be
	*  used at errors and therefore no issue for performance. Maybe one could
	*  export this few papaan::error-functions to an external class.
	*
	* @param object PEAR_Error $data
	* @see raiseError
	* @access public
	*/

	function handleError($data)
	{
		if (papaan::isError($data) && $data->getMode() == PEAR_ERROR_RETURN)
		{
            if (isset($this) && isset($this->errorLevel)) {
                $errorLevel = $this->errorLevel;
            // Global error handler
            } elseif (isset($GLOBALS['_PAPAAN_errorLevel'])) {
				$errorLevel = $GLOBALS['_PAPAAN_errorLevel'];
            }

			// if code is FATAL and mode is PEAR_ERROR_RETURN, die...
			// if mode is not PEAR_ERROR_RETURN, then the user likes to have the
			//  error mode handled differently and we don't die here
			 if ($data->getCode() & PAPAAN_ERROR_FATAL & $errorLevel)
			 {
				print('<b>Fatal Error (Papaan)</b>: '.$data->getMessage().'<br />');
				if (function_exists('debug_backtrace'))
				{
				$trace = debug_backtrace();
				print 'Backtrace: <br/>';

				foreach($trace as $ent)
				{
					if(isset($ent['file']))
						echo $ent['file'].':';
					if(isset($ent['function']))
					{
						echo $ent['function'].'(';
						if(isset($ent['args']))
						{
							$args='';
							foreach($ent['args'] as $arg)
							{
								$args.=$arg.',';
							}
							echo rtrim($args,',');
						}
						echo ') ';
					}
					if(isset($ent['line']))
						echo 'at line '.$ent['line'].' ';
					if(isset($ent['file']))
						echo 'in '.$ent['file']; echo "\n";
				}
				}
				die();
			 }
			 else if ($data->getCode() & PAPAAN_ERROR_WARNING & $errorLevel)
			 {
				print '<b>Warning (Papaan)</b>: '.$data->getMessage().'<br />';
			 }
			 else if ($data->getCode() & PAPAAN_ERROR_NOTICE & $errorLevel)
			 {
				print '<b>Notice (Papaan)</b>: '.$data->getMessage() .'<br />';
			 }

		}
	}

	/**
	* sets errorLevel for the papaan class
	*
	* It works like the php-error handling. You have to bit-operate the different
	*  papaan error-levels. At the moment these are
	*
	* PAPAAN_ERROR_NOTICE, PAPAAN_ERROR_WARNING, PAPAAN_ERROR_FATAL.
	*
	* You should not turn off WARNING and FATAL
	*
	* Default is PAPAAN_ERROR_DEFAULT = 7
	*
	* @param int $level	the wished level
    * @access public
	* @return int old errorLevel;
	*/
	function setErrorLevel($level)
	{
        if (isset($this)) {
			$oldlevel = $this->errorLevel;
            $this->errorLevel = $level;

        } else {
			$oldlevel = $GLOBALS['_PAPAAN_errorLevel'];
            $GLOBALS['_PAPAAN_errorLevel'] = $level;
        }
		return $oldlevel;
	}


}

function myErrorHandler ($errno, $errstr, $errfile, $errline) {
	echo "$errno: $errstr in $errfile at line $errline\n";
	if (function_exists('debug_backtrace'))
{
	echo 'Backtrace'."\n";
	$trace = debug_backtrace();
	foreach($trace as $ent) {
		if(isset($ent['file'])) echo $ent['file'].':';
		if(isset($ent['function']))
		{
			echo $ent['function'].'(';
			if(isset($ent['args']))
			{
				$args='';
				foreach($ent['args'] as $arg) { $args.=$arg.','; }
				echo rtrim($args,',');
			}
			echo ') ';
		}
		if(isset($ent['line'])) echo 'at line '.$ent['line'].' ';
		if(isset($ent['file'])) echo 'in '.$ent['file'];
		echo '<hr>';
	}
}
}

?>
