<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:map="http://apache.org/cocoon/sitemap/1.0"
>
    <xsl:output omit-xml-declaration="yes" indent="no" encoding="ISO-8859-1" method="xml"/>
    <xsl:include href="../common.xsl"/>

    <xsl:template match="/map:sitemap"><xsl:processing-instruction name="php">
	    $pipelineHit = false;
            <xsl:apply-templates select="map:components"/>
            <xsl:apply-templates select="map:pipelines"/>
	    return false;
    </xsl:processing-instruction></xsl:template>
    
    <xsl:template name="tag">
	<xsl:text>&lt;</xsl:text>
	<xsl:value-of select="name()"/>
	<xsl:for-each select="@*">
		<xsl:text> </xsl:text>
		<xsl:value-of select="name()"/>
		<xsl:text>="</xsl:text>
		<xsl:call-template name="escapeSingleQuotes">
		<xsl:with-param name="text" select="normalize-space(.)"/>
		</xsl:call-template>
		<xsl:text>"</xsl:text>
	</xsl:for-each>
	<xsl:if test="namespace-uri()">
		<xsl:text> xmlns</xsl:text>
		<xsl:if test="substring-before(name(),':')">
			<xsl:text>:</xsl:text>
			<xsl:value-of select="substring-before(name(),':')"/>
		</xsl:if>
		<xsl:text>="</xsl:text>
		<xsl:value-of select="namespace-uri()"/>
		<xsl:text>"</xsl:text>
	</xsl:if>
	<xsl:choose>
		<xsl:when test="*|text()">
			<xsl:text>&gt;</xsl:text>
			<xsl:apply-templates select="*|text()"/>
			<xsl:text>&lt;/</xsl:text>
			<xsl:value-of select="name()"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>/</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>&gt;</xsl:text>
    </xsl:template>

    <xsl:template match="map:matchers/map:matcher|map:serializers/map:serializer|map:generators/map:generator|map:readers/map:reader|map:transformers/map:transformer|map:selectors/map:selector|map:actions/map:action">
	$this->components['<xsl:value-of select="local-name()"/>']['<xsl:value-of select="@name"/>'] = array(
        <xsl:for-each select="@*[name()!='name']">'<xsl:value-of select="name()"/>'=><xsl:call-template name="translateString"><xsl:with-param name="text" select="."/></xsl:call-template>,</xsl:for-each>
	<xsl:if test="*">
	'dom' =>
	'&lt;config&gt;<xsl:apply-templates select="*"/>&lt;/config&gt;'
	</xsl:if>);
    </xsl:template>

    <xsl:template match="map:matcher//text()|map:serializer//text()|map:generator//text()|map:reader//text()|map:transformer//text()|map:selector//text()|map:action//text()">
	<xsl:call-template name="escapeSingleQuotes">
	<xsl:with-param name="text" select="normalize-space(.)"/>
	</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:matcher//*|map:serializer//*|map:generator//*|map:reader//*|map:transformer//*|map:selector//*|map:action//*">
    	<xsl:call-template name="tag"/>
    </xsl:template>

    <xsl:template match="map:components/map:matchers|map:components/map:serializers|map:components/map:generators|map:components/map:transformers|map:components/map:readers|map:components/map:selectors|map:components/map:actions">
	$this->default_components['<xsl:value-of select="local-name()"/>'] = '<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="@default"/></xsl:call-template>';
        <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="map:sitemap/map:pipelines|map:sitemap/map:components">
        <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="map:pipelines/map:pipeline/map:parameter[@name = 'expires']">    
        <xsl:if test="starts-with(@value,'access plus')">
            $this->setHeader('Expires',gmdate('r',strtotime('<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="substring-after(@value,'access plus')"/></xsl:call-template>')));
        </xsl:if>

    </xsl:template>
    
    <xsl:template match="map:pipelines/map:pipeline">
        
        <xsl:if test="@benchmark">
            $this->benchmark['do'] = True;
        </xsl:if>
    
        <xsl:choose>
	   <xsl:when test="@cachable = 'yes' or @cachable = 'true'">
              $this->enableCaching(<xsl:call-template name="generateAttributes"/>);
           </xsl:when>
           <xsl:otherwise>
              $this->disableCaching();
           </xsl:otherwise>
        </xsl:choose>
        
        <xsl:apply-templates/>	

	if ($pipelineHit) {
		return true;
	}
        
    </xsl:template>

	<xsl:template match="map:pipeline//map:mount|map:resource//map:mount">
		if($this->_mount(<xsl:call-template name="generateAttributes"/>)) {
			return true;
		}
	</xsl:template>		

	<xsl:template match="map:pipeline//map:aggregate|map:resource//map:aggregate">
	   	include_once(PAPAAN_BASE.'/components/aggregator.php');
        $aggregator = &amp;new aggregator($this);
		$aggregator->init(<xsl:call-template name="generateAttributes"/>);
		<xsl:apply-templates/>
		$aggregator->start($this->xml);
	</xsl:template>		
	
	<xsl:template match="map:aggregate/map:part">
		$aggregator->addPart(<xsl:call-template name="generateAttributes"/>);
	</xsl:template>		

    <xsl:template match="map:pipeline//map:generate|map:resource//map:generate" name="doGenerate">
        <xsl:param name="xmlVar" select="'$this->xml'"/>
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">generator</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>						
        </xsl:call-template>

                if($this->componentCache AND $this->componentCache->init($generator)){
                                                            
                    if(!$this->componentCache->isCached()){
                        /* miss! :( */
                        $generator->DomStart(<xsl:value-of select="$xmlVar"/>);          
                        $this->componentCache->store(<xsl:value-of select="$xmlVar"/>);
                    }
                    else{
                        /* hit! :)  */
                    }
                }
                else $generator->DomStart(<xsl:value-of select="$xmlVar"/>);
            
                $pipelineHit = True;
    </xsl:template>

    <xsl:template match="map:pipeline//map:read|map:resource//map:read">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">reader</xsl:with-param>
        </xsl:call-template>

        if($this->componentCache AND $this->componentCache->init($reader)){

            if(!$componentCache->isCached()) {
                $this->componentCache->store(null, 'start', null, true);
            }
            else{
                print($componentCache->load());
            }
        
        }
        else{
            $reader->start();
        }
        		
        $pipelineHit = True;
	return true;
        
    </xsl:template>

    
    <xsl:template match="map:pipeline//map:transform|map:resource//map:transform">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">transformer</xsl:with-param>			
            <xsl:with-param name="doParams">true</xsl:with-param>						
        </xsl:call-template>
        
         if($this->componentCache AND $this->componentCache->init($transformer)){
                if(!$this->componentCache->isCached()){
                        $this->componentCache->loadLast($this->xml);
                        $transformer->DomStart($this->xml);
                        $this->componentCache->store($this->xml);                   
                 }
                   
                else {}
          }
         else{
                $transformer->DomStart($this->xml);
             }
        
        
    </xsl:template>
    
	<xsl:template match="map:act/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">action</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>						
		</xsl:call-template>
    </xsl:template>


    <xsl:template match="map:select/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">selector</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:transform/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">transformer</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:generate/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">generator</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:serialize/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">serializer</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

	<xsl:template match="map:pipeline//map:act|map:resource//map:act">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">action</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>
            <xsl:with-param name="doInit">
	    <xsl:choose>
	    <xsl:when test="@src">true</xsl:when>
	    <xsl:otherwise>false</xsl:otherwise>
	    </xsl:choose>
	    </xsl:with-param>
        </xsl:call-template>

        if ($map = $action->act())
        {
			$this->addMap($map);
		        <xsl:apply-templates select="*[name() != 'map:parameter']"/>		
			$this->removeMap();
        } 


	</xsl:template>
    
    <xsl:template match="map:pipeline//map:serialize|map:resource//map:serialize">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">serializer</xsl:with-param>			
            <xsl:with-param name="doParams">true</xsl:with-param>						
            <xsl:with-param name="doInit">false</xsl:with-param>						
        </xsl:call-template>

                if($this->componentCache AND $this->componentCache->init($serializer)){
    
                    if($this->componentCache->isCached()){
                        $this->componentCache->load(true);
                    }
                    else{
                        $this->componentCache->loadLast($this->xml);
                        $this->componentCache->store(null, 'DomStart', $this->xml, true);
                        } 
                }

                else{
		   // print headers only if internal request
		   if (!$this->internal) {
	                   $this->printHeader();
		   }
                   $serializer->DomStart($this->xml);
                }
		return true;

        
    </xsl:template>
    
    <xsl:template match="map:match">
		<xsl:call-template name="setupComponent">
			<xsl:with-param name="prefix">matcher</xsl:with-param>
            <xsl:with-param name="doInit">false</xsl:with-param>						
		</xsl:call-template>

        if ($matcher->match(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@pattern"/></xsl:call-template>))
        {
        <xsl:apply-templates/>
        } 
        // end of map match = <xsl:value-of select="@pattern"/>
        
        $this->removeMap();

    </xsl:template>
    
    <xsl:template match="map:select">
		<xsl:call-template name="setupComponent">
			<xsl:with-param name="prefix">selector</xsl:with-param>
            		<xsl:with-param name="doParams">true</xsl:with-param>
            <xsl:with-param name="doInit">false</xsl:with-param>						
		</xsl:call-template>

        <xsl:apply-templates select="map:when|map:otherwise"/>
		$selector = null;
    </xsl:template>
    
    <xsl:template match="map:select//map:when">
        
        <xsl:if test="position() > 1">
        else</xsl:if> if ($selector->test(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@test"/></xsl:call-template>)) {

        <xsl:apply-templates />
        } // end when test = <xsl:value-of select="@test"/>
    </xsl:template>
    
    <xsl:template match="map:select//map:otherwise">
        else {
        <xsl:apply-templates />
        } // end else
    </xsl:template>
    

    <xsl:template match="map:pipeline//map:redirect-to|map:resource//map:redirect-to">
        $this->redirectTo(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@uri"/></xsl:call-template>);
	return true;
    </xsl:template>


    <xsl:template match="map:action[@type='disableOutputCaching']">
        $this->disableOutputCaching();
    </xsl:template>
    
    <xsl:template name="generateAttributes">
        array(
        <xsl:for-each select="@*">'<xsl:value-of select="name()"/>'=><xsl:call-template name="translateString"><xsl:with-param name="text" select="."/></xsl:call-template>,</xsl:for-each>
        )
    </xsl:template>
    
    <xsl:template match="*">
        papaan::raiseError('<xsl:value-of select="name()"/> is not allowed as a child of <xsl:value-of select="name(..)"/> in your sitemap',PAPAAN_ERROR_WARNING);
        <xsl:apply-templates/>
    </xsl:template>   
    
    <xsl:template name="setupComponent">
        <xsl:param name="prefix"/>
	<xsl:param name="doParams">false</xsl:param>
	<xsl:param name="doInit">true</xsl:param>

	$<xsl:value-of select="$prefix"/> = &amp;$this->new_component('<xsl:value-of select="$prefix"/>',<xsl:call-template name="translateString"><xsl:with-param name="text" select="@type"/></xsl:call-template>);
		
	<xsl:if test="$doParams != 'false'">
	<xsl:apply-templates select="map:parameter"/>
	</xsl:if>
	<xsl:if test="$doInit != 'false'">
        $<xsl:value-of select="$prefix"/>->init(<xsl:call-template name="generateAttributes"/>);
	</xsl:if>
    </xsl:template>
	
	

	<xsl:template name="setParameter">
		<xsl:param name="prefix"/>
		$<xsl:value-of select="$prefix"/>->setParameter(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@name"/></xsl:call-template>,<xsl:call-template name="translateString"><xsl:with-param name="text" select="@value"/></xsl:call-template>);
		</xsl:template>
    
    <xsl:template name="map">
	<xsl:param name="path"/>
	<xsl:text>$this->maps[0]['</xsl:text>
	<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$path"/></xsl:call-template>
	<xsl:text>']</xsl:text>
    </xsl:template>

    <xsl:template name="translateString">
	<xsl:param name="text"/>
    	<xsl:choose>
		<xsl:when test="$text != ''">
			<xsl:choose>
				<xsl:when test="contains($text,'{') and contains($text,'}')">
					<xsl:text>$this->translateScheme('</xsl:text>
					<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$text"/></xsl:call-template>
					<xsl:text>')</xsl:text>
					<!--
					<xsl:if test="substring-before($text,'{')">
						<xsl:text>'</xsl:text>
						<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="substring-before($text,'{')"/></xsl:call-template>
						<xsl:text>' . </xsl:text>
					</xsl:if>
					<xsl:call-template name="map"><xsl:with-param name="path" select="substring-before(substring-after($text,'{'),'}')"/></xsl:call-template>
					<xsl:if test="substring-after($text,'}')">
						<xsl:text> . </xsl:text>
						<xsl:call-template name="translateString"><xsl:with-param name="text" select="substring-after($text,'}')"/></xsl:call-template>
					</xsl:if>
					-->
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>'</xsl:text>
					<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$text"/></xsl:call-template>
					<xsl:text>'</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
		<xsl:text>''</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
    </xsl:template>

    <xsl:template match="map:pipeline//map:call|map:resource//map:call">
	<xsl:variable name="resource" select="@resource"/>
	$tmp = array();
	<xsl:apply-templates select="*"/>
	$this->addMap($tmp);
	<xsl:apply-templates select="/map:sitemap/map:resources/map:resource[@name=$resource]/*"/>
	$this->removeMap();
    </xsl:template>

    <xsl:template match="map:call/map:parameter">
	$tmp[<xsl:call-template name="translateString"><xsl:with-param name="text" select="@name"/></xsl:call-template>] = <xsl:call-template name="translateString"><xsl:with-param name="text" select="@value"/></xsl:call-template>;
    </xsl:template>

</xsl:stylesheet>
