<?php
// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Mike Hommey <mh@glandium.org>                                |
// +----------------------------------------------------------------------+
//
// $Id: form_validator.php,v 1.1 2003/07/23 20:54:45 glandium Exp $

include_once(PAPAAN_BASE."/components/action.php");
/**
* Far from perfection form validator
* @author   Mike Hommey <mh@glandium.org>
* @version  $Id: form_validator.php,v 1.1 2003/07/23 20:54:45 glandium Exp $
* @package  papaan
*/

class action_form_validator extends action {

	/**
    * Constructor
    *
	*/
	function action_form_validator(&$sitemap, $config) {
		$this->action($sitemap,$config);
	}

	function act() {
		$validate = $this->getParameter('validate-set');
		if (! $validate) {
			$validate = $this->getParameter('validate');
			if ($validate) {
				$validate = split(',',$validate);
			}
		}
		if (! $validate) {
			$this->sitemap->raiseError($this,'error','At least one of validate or validate-set parameter must be defined');
		}
		$descriptor = $this->readfile($this->getParameter('descriptor'));
		return $this->validate_form($descriptor, $validate);
	}

	function validate_form($descriptor, $validate = null) {
		$xpath = xpath_new_context($descriptor);
		if (is_array($validate)) {
			foreach($validate as $name) {
				$xnode = xpath_eval($xpath, '/*/parameter[@name=\''.$name.'\']');
				foreach($xnode->nodeset as $node) {
					if (!$this->check_param($node)) {
						return false;
					}
				}
			}
		} else {
			$xnode = xpath_eval($xpath, '/*/constraint-set[@name=\''.$validate.'\']/*');
			foreach($xnode->nodeset as $vnode) {
				$node = xpath_eval($xpath, '/*/parameter[@name=\''.$vnode->get_attribute('name').'\']');
				foreach($node->nodeset[0]->attributes() as $attrib) {
					if (!$vnode->has_attribute($attrib->name())) {
						$vnode->set_attribute($attrib->name(),$attrib->value());
					}
				}
				if (!$this->check_param($vnode)) {
					return false;
				}
			}
		}
	}

	function check_param($node) {
		$name = $node->get_attribute('name');
		$value = $this->sitemap->request->get_parameter($name);
		if (!$value) {
			$nullable = $node->get_attribute('nullable');
			if ($nullable != 'yes') {
				return false;
			}
			$default = $node->get_attribute('default');
			if ($default !== null) {
				$this->sitemap->request->parameters[$name] = $default;
			}
			return true;
		}
		$type = $node->get_attribute('type');
		if ($type == 'string') {
			$one_of = $node->get_attribute('one-of');
			if ($one_of) {
				if (in_array($value, split('|',$one_of))) {
					return true;
				}
				return false;
			}
			$matches_regex = $node->get_attribute('matches-regex');
			if ($matches_regex && !preg_match('/'.$matches_regex.'/',$value)) {
				return false;
			}
			$min_len = $node->get_attribute('min-len');
			if (($min_len !== '') && (strlen($value) < $min_len)) {
				return false;
			}
			$max_len = $node->get_attribute('max-len');
			if (($max_len !== '') && (strlen($value) > $max_len)) {
				return false;
			}
			return true;
		}
		if ($type == 'long' || $type == 'double') { // should check long/double
			if (!is_numeric($value)) {
				return false;
			}
			$min = $node->get_attribute('min');
			if (($min !== '') && ($value < $min)) {
				return false;
			}
			$max = $node->get_attribute('max');
			if (($max !== '') && ($value > $max)) {
				return false;
			}
			return true;
		}
		return false;
	}
}
