<?php
// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// | Copyright (c) 2003 Mike Hommey                                       |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Mike Hommey <mh@glandium.org>                                |
// +----------------------------------------------------------------------+
//
// $Id: directory.php,v 1.2 2003/07/17 18:32:03 glandium Exp $

include_once(PAPAAN_BASE.'/components/generator.php');

/**
* This class returns xml from directory listing
*
* @author   Mike Hommey <mh@glandium.org>
* @version  $Id: directory.php,v 1.2 2003/07/17 18:32:03 glandium Exp $
* @package  papaan
*/
class generator_directory extends generator {


    /**
    * Constructor, does nothing at the moment
    */
    function generator_directory (&$sitemap, $config) {
        $this->generator($sitemap, $config);
    }

    /**
    * generates an xml-DomDocument out of the xml-file
    *
    * @access public
    * @returns object DomDocument XML-Document
    */
    function DomStart(&$xml)
    {
	$src = $this->getAttrib('src');
	$xml = domxml_new_doc('1.0');
	$root = $xml->create_element_ns('http://apache.org/cocoon/directory/2.0','directory','dir');
	$xml->add_child($root);
	if ($this->read_directory($xml,$root,$this->sitemap->base_dir.'/'.$src, $this->getParameter('depth'))) {
		$stat = stat($this->sitemap->base_dir.'/'.$src);
		$root->set_attribute('size',$stat['size']);
		$root->set_attribute('lastModified',$stat['mtime']*1000);
	} else {
		$xml = NULL;
	}
        return True;
    }

    function read_directory($dom, $parent, $directory, $depth = 1) {
	if (is_dir($directory) && ($dh = opendir($directory))) {
		while (($file = readdir($dh)) !== false) {
			if (($file != ".") && ($file != "..")) {
				if (is_file($directory.'/'.$file)) {
					$node = $dom->create_element('file');
				} else if (is_dir($directory.'/'.$file)) {
					$node = $dom->create_element('directory');
				}
				$node->set_attribute('name', $file);
				$stat = stat($directory.'/'.$file);
				$node->set_attribute('size',$stat['size']);
				$node->set_attribute('lastModified',$stat['mtime']*1000);
				$parent->add_child($node);
				$node->set_namespace('http://apache.org/cocoon/directory/2.0');
				if (is_dir($directory.'/'.$file) and ($depth > 1)) {
					$this->read_directory($dom, $node, $directory.'/'.$file,$depth - 1);
				}
			}
		}
		return true;
	} else {
		return false;
	}
    }
    
    /* CACHING STUFF */

    /**
     * Generate cacheKey
     *
     * Calls the method inherited from 'Component'
     *
     * @param   array  attributes
     * @param   int    last cacheKey
     * @see     generateKeyDefault()
     */
    function generateKey($attribs, $keyBefore){
        return($this->generateKeyDefault($attribs, $keyBefore));
    }

    /** Generate validityObject  
     *
     * This is common to all "readers", you'll find the same code there.
     * I'm thinking about making a method in the class component named generateValidityFile() or alike
     * instead of having the same code everywhere..
     *
     * @author Hannes Gassert <hannes.gassert@unifr.ch>
     * @see  checkvalidity()
     * @return  array  $validityObject contains the components attributes plus file modification time and time of last access.
     */
    function generateValidity(){
        $validityObject = $this->attribs;
        $src = $this->getAttrib('src');
        $validityObject['filemtime'] = filemtime($src);
        $validityObject['fileatime'] = fileatime($src);
        return($validityObject);
    }

    /**
     * Check validity of a validityObject from cache
     *
     * This implements only the most simple form: If there's no fresher version, take that from cache.
     * I guess we'll need some more refined criteria..
     *
     * @return  bool  true if the validityObject indicates that the cached version can be used, false otherwise.
     * @param   object  validityObject
     */
    function checkValidity($validityObject){
        return(isset($validityObject['src'])       &&
               isset($validityObject['filemtime']) &&
               file_exists($validityObject['src']) &&
               ($validityObject['filemtime'] == filemtime($validityObject['src'])));
    }

}


?>
